@@ -1,114 +1,38 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from decimal import Decimal, ROUND_DOWN, ROUND_HALF_EVEN
+from decimal import ROUND_DOWN, ROUND_HALF_EVEN, Decimal
 from operator import itemgetter
 
 from trytond.model import ModelView, Workflow, fields
-from trytond.pyson import Eval, Bool
-from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
-
 from trytond.modules.product import price_digits, round_price
+from trytond.modules.stock_shipment_cost import ShipmentCostMixin
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
 
 
-class ShipmentIn(metaclass=PoolMeta):
+class ShipmentIn(ShipmentCostMixin, metaclass=PoolMeta):
     __name__ = 'stock.shipment.in'
-    carrier = fields.Many2One('carrier', 'Carrier', states={
-            'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
 
-    cost_currency_used = fields.Function(fields.Many2One(
-            'currency.currency', "Cost Currency",
-            states={
-                'invisible': Eval('cost_edit', False),
-                },
-            depends=['cost_edit']),
-        'on_change_with_cost_currency_used')
-    cost_currency = fields.Many2One(
-        'currency.currency', "Cost Currency",
-        states={
-            'required': Bool(Eval('cost')),
-            'invisible': ~Eval('cost_edit', False),
-            'readonly': ~Eval('state').in_(['draft']),
-            },
-        depends=['cost', 'cost_edit', 'state'])
-    cost_used = fields.Function(fields.Numeric(
-            "Cost", digits=price_digits,
-            states={
-                'invisible': Eval('cost_edit', False),
-                },
-            depends=['cost_edit']),
-        'on_change_with_cost_used')
-    cost = fields.Numeric(
-        "Cost", digits=price_digits,
-        states={
-            'invisible': ~Eval('cost_edit', False),
-            'readonly': ~Eval('state').in_(['draft']),
-            },
-        depends=['cost_edit', 'state'])
+    @fields.depends('state')
+    def on_change_with_shipment_cost_readonly(self, name=None):
+        return self.state != 'draft'
 
-    cost_edit = fields.Boolean(
-        "Edit Cost",
-        states={
-            'readonly': ~Eval('state').in_(['draft']),
-            },
-        depends=['state'],
-        help="Check to edit the cost.")
-
-    def _get_carrier_context(self):
-        return {}
-
-    @fields.depends('carrier', methods=['_get_carrier_context'])
-    def _compute_costs(self):
-        costs = {
-            'cost': None,
-            'cost_currency': None,
-            }
-        if self.carrier:
-            with Transaction().set_context(self._get_carrier_context()):
-                cost, currency_id = self.carrier.get_purchase_price()
-            costs['cost'] = round_price(cost)
-            costs['cost_currency'] = currency_id
-        return costs
-
-    @fields.depends(
-        'state', 'cost_currency', 'cost_edit', methods=['_compute_costs'])
-    def on_change_with_cost_currency_used(self, name=None):
-        if (not self.cost_edit
-                and self.state not in {'cancelled', 'received', 'done'}):
-            return self._compute_costs()['cost_currency']
-        elif self.cost_currency:
-            return self.cost_currency.id
-
-    @fields.depends('state', 'cost', 'cost_edit', methods=['_compute_costs'])
-    def on_change_with_cost_used(self, name=None):
-        if (not self.cost_edit
-                and self.state not in {'cancelled', 'received', 'done'}):
-            return self._compute_costs()['cost']
-        else:
-            return self.cost
-
-    @fields.depends(
-        'cost_edit', 'cost_used', 'cost_currency_used')
-    def on_change_cost_edit(self):
-        if self.cost_edit:
-            self.cost = self.cost_used
-            self.cost_currency = self.cost_currency_used
+    @property
+    def shipment_cost_moves(self):
+        return [
+            m for m in self.incoming_moves
+            if m.state not in {'done', 'cancelled'}]
 
     def allocate_cost_by_value(self):
         pool = Pool()
         Currency = pool.get('currency.currency')
         Move = pool.get('stock.move')
 
-        if not self.cost_used:
+        cost = self._get_shipment_cost()
+        if not cost:
             return
 
-        cost = Currency.compute(
-            self.cost_currency_used, self.cost_used, self.company.currency,
-            round=False)
-        moves = [m for m in self.incoming_moves
-            if m.state not in ('done', 'cancelled')]
+        moves = self.shipment_cost_moves
 
         sum_value = 0
         unit_prices = {}
@@ -187,8 +111,11 @@ class ShipmentIn(metaclass=PoolMeta):
 
 class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
-    unit_shipment_cost = fields.Numeric('Unit Shipment Cost',
-        digits=price_digits, readonly=True)
+    unit_shipment_cost = fields.Numeric(
+        "Unit Shipment Cost", digits=price_digits, readonly=True,
+        states={
+            'invisible': ~Eval('unit_shipment_cost'),
+            })
 
     def _compute_unit_price(self, unit_price):
         if self.unit_shipment_cost:
@@ -246,8 +173,9 @@ class Move(metaclass=PoolMeta):
                 moves, quantity, type_):
             if (type_.startswith('in_')
                     and move.unit_shipment_cost):
-                shipment_cost = Uom.compute_price(move.uom,
-                    move.unit_shipment_cost, move.product.default_uom)
+                shipment_cost = Uom.compute_price(
+                    move.unit, move.unit_shipment_cost,
+                    move.product.default_uom)
                 shipment_cost = Currency.compute(move.currency,
                     shipment_cost, move.company.currency)
                 cost_price -= shipment_cost
